#include <stdio.h>

/* copy(a,b,n) copies an array a of length n
 in an array b. */
void copy(int a[], int b[], int n) {
 int i;
 for (i = 0 ; i < n ; i++) {
  b[i] = a[i];
 }
}

/* Returns 1 when 2 arrays a and b are equal, 0 either. */
int isEqual(int a[], int b[], int n) {
 int i;
 for (i = 0 ; i < n ; i++) {
  if (b[i] != a[i]) return 0;
 }
 return 1;
}

/* Returns 1 when the array a is lower than the array b
 in the lexicographic order induced by the precedence
 order < on the integers, 0 otherwise. */
int isLexlt(int a[], int b[], int n) {
 int i = 0;
 while (i < n) {
  if (a[i] == b[i]) i++;
  else {
   if (a[i] < b[i]) return 1;
   else return 0;
  }
 }
 return 0;
}


// Display an array of length n.
void displayl(int l[], int n) {
 int i;
 printf("[ ");
 for (i = 0 ; i < n ; i++) {
  printf("%d ", l[i]);
 }
 printf("]");
}

///////////////// count ////////////////////////

// Computes n!
int fact (int n) {
 int i;
 int x = 1;
 for (i=2;i<=n;i++) {
  x = x*i; 
 }
 return x;
}
        
// Computes x^n
int power (int x, int n) {
   int y = 1;
   int m = 1;
   while (m <= n) {
      y *= x;
      m++;
   }
   return y;
}

/* count108(n) computes the n-th Catalan number, for n <= MAX.
 * See http://oeis.org/A000108 for details.
 * Warning: Arithmetic overflow for n >= MAX + 1. */
int count108(int n) {
 if (n < 0)
  return -1;
 else if (n == 0)
  return 1;
 else
  return 2*(2*n-1)*count108(n-1)/(n+1);
}

/* count85(n) computes the n-th number of the
 * sequence https://oeis.org/A000085. */
int count85(int n) {
 if (n < 0)
  return -1;
 else if (n <= 1)
       return 1;
      else if (n == 2)
            return 2;
           else return count85(n-1) + (n-1)*count85(n-2);
}

/* count166(n) computes the n-th number of the 
 * sequence https://oeis.org/A000166. */
int count166(int n) {
 if (n < 0)
  return -1;
 else if (n == 0)
       return 1;
      else return n*count166(n-1) + power(-1,n);
}


/* binomial(n,k) computes the number of ways to choose k elements from 
 a set of n elements, equals the binomial coefficient ‘n choose k'. */
int binomial(int n, int k) { 
 int b,f,j;
 if(k > n) return 0;
 if ((k == 0) || (k == n)) return 1;
 if(2*k > n) k = n-k; 
 b = n-k+1;
 f= b;
 for (j = 2; j <= k; ++j){
  f++; 
  b *= f; 
  b /= j;
 } 
 return b;
}

/* Returns the number of injections from [0..n-1] to [0..k]. */
int countInj(int n, int k) { 
 if(k < n) return 0;
 return binomial(k,n)*fact(n);
}

/* Returns the number of surjections from [0..n-1] to [0..p-1]. */
int countSurj(int n, int p) {
 if (n <= 0 || p <= 0 || p > n) return 0;
 if (n == 1 && p == 1) return 1;
 // 2 <= n, 1 <= p <= n
 return(p*(countSurj(n-1,p-1)+countSurj(n-1,p)));
}


//////////////////// conversion ///////////////////////////

/* displaySubset() display the subsets of a set of n elements
   from the array ds of delta set. */
int delta2subset(int ds[], int s[], int n) {
 int i,c;
 c = 0;
 for (i = 0; i < n; i++) {
  if (ds[i] == 1) {
   s[c] = i;
   c++;
  }
 }
 return c;
}

