/* Generation algorithm for RGFs.
 Adaptation of the algorithm page 235 of the fxtbook. */

/*@ predicate is_non_neg(int *a, integer b, integer c) = (\forall integer i; b <= i < c ==> a[i] >= 0);
  @ predicate is_le_pred(int *a, integer b, integer c) = (\forall integer i; b <= i < c ==> a[i] <= a[i-1]+1);
  @ predicate is_RGF(int *a, integer n) = a[0] == 0 && is_non_neg(a,0,n) && is_le_pred(a,1,n); */

/*@ predicate is_zero(int *a, integer b, integer c) =
  @  (\forall integer i; b <= i < c ==> a[i] == 0); */

/*@ requires n > 0 && \valid(r+(0..n-1));
  @ assigns r[0..n-1];
  @ ensures is_RGF(r,n); */
void firstRGF(int r[], int n) {
 int k;
 /*@ loop invariant 0 <= k <= n;
   @ loop invariant is_zero(r,0,k);
   @ loop assigns k, r[0..n-1];
   @ loop variant n-k; */
 for (k = 0; k < n; k++) { r[k] = 0; }
}

/*@ requires n > 0 && \valid(r+(0..n-1));
  @ requires is_RGF(r,n);
  @ assigns r[1..n-1]; 
  @ ensures is_RGF(r,n); */
int nextRGF(int r[], int n) {
 int i,k;
 /*@ loop invariant 0 <= i <= n-1;
   @ loop assigns i;
   @ loop variant i-1; */
 for (i = n-1; i >= 1; i--)
  if (r[i] <= r[i-1]) { break; }
 if (i == 0) { return 0; } // Last RGF.
 r[i] = r[i] + 1;
 /*@ loop invariant i+1 <= k <= n;
   @ loop invariant is_non_neg(r,0,k);
   @ loop invariant is_le_pred(r,1,k);
   @ loop assigns k, r[i+1..n-1];
   @ loop variant n-k; */
 for (k = i+1; k < n; k++) { r[k] = 0; }
 return 1;
}
