/* Generation of all subsets of a set of n elements from delta sets, in
 * the lexicographic increasing order (on the delta sets) induced by the
 * precedence 0 < 1. Adaptation of the algorithm page 203 of the fxtbook. */

/*@ predicate is_dset(int *a, integer n) =
  @  \forall integer i; 0 <= i < n ==> (a[i] == 0 || a[i] == 1); */

/*@ requires n >= 0 && \valid(s+(0..n-1));
  @ assigns s[0..n-1];
  @ ensures is_dset(s,n); */
int firstSubset(int s[], int n) {
 int k;
 /*@ loop invariant 0 <= k <= n;
   @ loop invariant is_dset(s,k);
   @ loop assigns k, s[0..n-1];
   @ loop variant n-k; */
 for ( k = 0; k < n; k++) {
  s[k] = 0;
 }
 return 1;
}

/*@ requires n >= 0 && \valid(s+(0..n-1));
  @ requires is_dset(s,n);
  @ assigns s[0..n-1];
  @ ensures is_dset(s,n); */
int nextSubset(int s[], int n) {
 int i,k;

 /*@ loop invariant -1 <= k <= n-1;
   @ loop assigns k;
   @ loop variant k; */
 for (k = n-1; k >= 0; k--) { if (s[k] == 0) { break; } }
 if (k == -1) { return 0; }
 s[k] = 1;
 /*@ loop invariant k+1 <= i <= n;
   @ loop invariant is_dset(s,i);
   @ loop assigns i,s[k+1..n-1];
   @ loop variant n-i; */
 for (i = k+1; i < n; i++) { s[i] = 0; }
 return 1;
}
