/* Generation of all derangements of n elements. Here, by
 * filtering among all permutations of a set of n elements generated in
 * lexicographic order.
 */

#include "../fct/fct.h"
#include "../perm/perm.h"

/*@ predicate is_ff(int *a, integer n) =
  @  \forall integer i; 0 <= i < n ==> a[i] != i;
  @ predicate is_derang(int *a, integer n) = is_perm(a,n) && is_ff(a,n); */
  
/* Returns 1 when the array a (assumed to be a permutation) is a derangement
 * of n elements, and 0 otherwise. */
/*@ requires n > 0 && \valid(a+(0..n-1));
  @ requires is_perm(a,n);
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_derang(a,n); */
int isDerang(int a[], int n) {
 int i;
 
 /*@ loop invariant 0 <= i <= n;
   @ loop invariant is_ff(a,i);
   @ loop assigns i;
   @ loop variant n-i; */
 for (i = 0; i < n; i++) {
  if (a[i] == i) {
   return 0; 
  }
 }
 return 1;
}

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ requires is_derang(a,n);
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_derang(a,n); */
int nextDerang(int a[], int n) {
 int tmp;
 
 /*@ loop invariant is_perm(a,n);
   @ loop assigns a[0..n-1],tmp; */
 do {
  tmp = nextPerm(a,n);
     } while (tmp != 0 && isDerang(a,n) == 0);

 if (tmp == 0) { return 0; }
 return 1;
}

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_derang(a,n); */
int firstDerang(int a[], int n) {
 int tmp;
 
 tmp = firstPerm(a,n);
 
 /*@@ loop invariant tmp != 0 ==> is_perm(a,n);
   @ loop assigns a[0..n-1],tmp; */
 while (tmp != 0 && isDerang(a,n) == 0) {
  tmp = nextPerm(a,n);
 } 
 
 if (tmp == 0) { return 0; }
 return 1;
}



