/* Generation of all functions from [0..n-1] to [0..k-1], in the increasing
 * lexicographic order induced by the precedence order < on integers.
 * Each function is tabulated in an array of size n containing values between 0
 * and k-1. */

/*@ predicate is_fct(int *a, integer b, integer c) =
  @  \forall integer i; 0 <= i < b ==> 0 <= a[i] < c; */

/*@ requires 0 < n && 0 < k;
  @ requires \valid(f+(0..n-1));
  @ assigns f[0..n-1];
  @ ensures is_fct(f,n,k); */
int firstFct(int f[], int n, int k) {
 int i;
 /*@ loop invariant 0 <= i <= n;
   @ loop invariant is_fct(f,i,k);
   @ loop assigns i, f[0..n-1];
   @ loop variant n-i; */
 for (i = 0; i < n; i++) {
  f[i] = 0;
 }
 return 1;
}

/* Returns 1 if the array is modified, or 0 if the array
   is unchanged. */
/*@ requires 0 < n && 0 < k && \valid(f+(0..n-1));
  @ requires is_fct(f,n,k);
  @ assigns f[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures is_fct(f,n,k); */
int nextFct(int f[], int n, int k) {
 int i,j;

 // Find rightmost digit that can be incremented:
 /*@ loop invariant -1 <= j <= n-1;
   @ loop assigns j;
   @ loop variant j; */
 for (j = n-1; j >= 0; j--) {
  if (f[j] < k-1) { break; }
 }
 // If there is no such index, then f = [k-1,...,k-1] is the last fct.
 if (j == -1) { return 0; }
 f[j] = f[j] + 1;
 // Fill the suffix starting at f[j+1] with zeros:
 /*@ loop invariant j+1 <= i <= n;
   @ loop invariant is_fct(f,i,k);
   @ loop assigns i,f[j+1..n-1];
   @ loop variant n-i; */
 for (i = j+1; i < n; i++) { f[i] = 0; }
 return 1;
}
