/* Generation of all involutions of n elements. 
 Generation in increasing order. 
 Here, by filtering among all Generation of all permutations 
 of a set of n elements in lexicographic order.
 All is proved with alt-Ergo Fluorine and alt-Ergo 0.95.1.
 */


#include "../fct/fct.h"
#include "../perm/perm.h"

/*@ predicate is_inv(int *a, integer n) = 
  @  \forall integer i; 0 <= i < n ==> a[a[i]] == i;
  @ predicate is_invol(int *a, integer n) = 
  @  is_perm(a,n) && is_inv(a,n);
  @*/

// proved
/** Returns 1 when the array a is an involution 
  of n elements, and 0 otherwise. */
 /*@ requires n > 0 && \valid(a+(0..n-1));
   @ requires is_perm(a,n);
   @ assigns \nothing;
   @ ensures \result == 0 || \result == 1;
   @ ensures \result == 1 ==> is_invol(a,n); */
int isInvol(int a[], int n) {
 int i;
 
 /*@ loop invariant 0 <= i <= n;
   @ loop invariant is_inv(a,i);
   @ loop assigns i; 
   @ loop variant n-i;*/
 for (i = 0; i < n; i++) {
  if (a[a[i]] != i) {
   return 0;
  } 
 }
 return 1;
}

// proved
/*@ requires n > 0 && \valid(a+(0..n-1));
  @ requires is_invol(a,n);
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_invol(a,n); */
int nextInvol(int a[], int n) {
 int tmp;
 
 /*@ loop invariant is_perm(a,n);
   @ loop assigns a[0..n-1],tmp; */
 do {
  tmp = nextPerm(a,n);
     } while (tmp != 0 && isInvol(a,n) == 0);

 if (tmp == 0) { return 0; }
 return 1;
}

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_invol(a,n); */
int firstInvol(int a[], int n) {
 int tmp;
 
 tmp = firstPerm(a,n);
 
 /*@@ loop invariant tmp != 0 ==> is_perm(a,n);
   @ loop assigns a[0..n-1],tmp; */
 while (tmp != 0 && isInvol(a,n) == 0) {
  tmp = nextPerm(a,n);
 } 
 
 if (tmp == 0) { return 0; }
 return 1;
}


