/*@ axiomatic preds {
  @  predicate is_XXX(int *a, integer n)
  @   reads a[0..n-1];
  @  predicate is_YYY(int *a, integer n)
  @   reads a[0..n-1];
  @  predicate is_ZZZ(int *a, integer n) =
  @   is_XXX(a,n) && is_YYY(a,n);
  @ } */

/* Returns 1 when the array a satisfies
 * property is_YYY, and 0 otherwise. */
/*@ requires n > 0 && \valid(a+(0..n-1));
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 <==> is_YYY(a,n); */
int isYYY(int a[], int n);

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_XXX(a,n); */
int firstXXX(int a[], int n);

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ requires is_XXX(a,n);
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_XXX(a,n); */
int nextXXX(int a[], int n);

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_ZZZ(a,n); */
int firstZZZ(int a[], int n) {
 int tmp;

 tmp = firstXXX(a,n);
 /*@ loop invariant tmp != 0 ==> is_XXX(a,n);
   @ loop assigns a[0..n-1],tmp; */
 while (tmp != 0 && isYYY(a,n) == 0) {
  tmp = nextXXX(a,n);
 }
 if (tmp == 0) { return 0; }
 return 1;
}

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ requires is_ZZZ(a,n);
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_ZZZ(a,n); */
int nextZZZ(int a[], int n) {
 int tmp;

 /*@ loop invariant is_XXX(a,n);
   @ loop assigns a[0..n-1],tmp; */
 do {
  tmp = nextXXX(a,n);
 } while (tmp != 0 && isYYY(a,n) == 0);
 if (tmp == 0) { return 0; }
 return 1;
}
