/* Generation of all permutations on [0..n-1], tabulated in an array [0..n-1]
 * in the increasing lexicographic order induced by the precedence order < on
 * integers. Here, by filtering of endofunctions on [0..n-1] which are
 * surjections.
 *
 * File generated by instantiation of the patterns allex.c and filtering.c.
 *
 * Proved with Frama-C Neon and Alt-Ergo 0.95.1 or CVC3. */

#include "../fct/fct.h"
#include "../endofct/endofct.h"

/*@ predicate is_X3(int *a, integer n, integer n1, integer n2) = a[n2] == n1;
  @ predicate is_eq_im_gen(int *a, integer n, integer n1, integer n2) = \exists integer i2; 0 <= i2 < n2 && is_X3(a,n,n1,i2);
  @ predicate is_eq_im(int *a, integer n, integer n1) = is_eq_im_gen(a,n,n1,n);
  @ predicate is_im_gen(int *a, integer n, integer n1) = \forall integer i1; 0 <= i1 < n1 ==> is_eq_im(a,n,i1);
  @ predicate is_im(int *a, integer n) = is_im_gen(a,n,n); 
  @ predicate is_surj(int *a, integer n) = is_endofct(a,n) && is_im(a,n); */

/*@ requires n >= 0 && \valid(a+(0..n-1));
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 <==> is_X3(a,n,n1,n2);*/
int isX3(int a[], int n, int n1, int n2) {
 return(a[n2] == n1);
}

/*@ requires n >= 0 && \valid(a+(0..n-1));
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 <==> is_eq_im(a,n,n1);*/
int isEqIm(int a[], int n, int n1) {
 int i;

 /*@ loop invariant 0 <= i <= n;
   @ loop invariant ! is_eq_im_gen(a,n,n1,i);
   @ loop assigns i;
   @ loop variant n-i;*/
 for (i = 0; i < n; i++)
  if (isX3(a,n,n1,i) == 1) return 1;
 return 0;
}

/*@ requires n >= 0 && \valid(a+(0..n-1));
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 <==> is_im(a,n);*/
int isIm(int a[], int n) {
 int i;

 /*@ loop invariant 0 <= i <= n;
   @ loop invariant is_im_gen(a,n,i);
   @ loop assigns i;
   @ loop variant n-i;*/
 for (i = 0; i < n; i++)
  if (isEqIm(a,n,i) == 0) return 0;
 return 1;
}

/* Returns 1 when the array a is a surjection, and 0 otherwise. */
/*@ requires n > 0 && \valid(a+(0..n-1));
  @ requires is_endofct(a,n);
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_surj(a,n); */
int isSurjEndo(int a[], int n) {
 return (isIm(a,n));
}

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_surj(a,n); */
int firstSurjEndo(int a[], int n) {
 int tmp;
 
 tmp = firstEndofct(a,n);
 /*@ loop invariant tmp != 0 ==> is_endofct(a,n);
   @ loop assigns a[0..n-1],tmp; */
 while (tmp != 0 && isSurjEndo(a,n) == 0) {
  tmp = nextEndofct(a,n);
 } 
 if (tmp == 0) { return 0; }
 return 1;
}

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ requires is_surj(a,n);
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_surj(a,n); */
int nextSurjEndo(int a[], int n) {
 int tmp;
 
 /*@ loop invariant is_endofct(a,n);
   @ loop assigns a[0..n-1],tmp; */
 do {
  tmp = nextEndofct(a,n);
 } while (tmp != 0 && isSurjEndo(a,n) == 0);
 if (tmp == 0) { return 0; }
 return 1;
}
