/* Generation of all surjections from [0..n-1] to [0..k-1], tabulated in an array
 * [0..n-1] containing values between 0 and k-1, in the increasing lexicographic 
 * order induced by the precedence order < on integers.
 * Here, by filtering among all functions from [0..n-1] to [0..k-1].
 *
 * File generated by instantiation of the patterns allex.c and filtering.c.
 *
 * Proved with Frama-C Neon and Alt-Ergo 0.95.1 or CVC3. */

#include "../fct/fct.h"

/*@ predicate is_X3(int *a, integer n, integer k, integer n1, integer n2) = a[n2] == n1;
  @ predicate is_eq_im_gen(int *a, integer n, integer k, integer n1, integer n2) =
  @  \exists integer i2; 0 <= i2 < n2 && is_X3(a,n,k,n1,i2);
  @ predicate is_eq_im(int *a, integer n, integer k, integer n1) = is_eq_im_gen(a,n,k,n1,n);
  @ predicate is_im_gen(int *a, integer n, integer k, integer n1) = \forall integer i1; 0 <= i1 < n1 ==> is_eq_im(a,n,k,i1);
  @ predicate is_im(int *a, integer n, integer k) = is_im_gen(a,n,k,k);
  @ predicate is_surj(int *a, integer n, integer k) = is_fct(a,n,k) && is_im(a,n,k); */
  
  
/*@ requires n > 0 && \valid(a+(0..n-1));
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 <==> is_X3(a,n,k,n1,n2);*/
int isX3(int a[], int n, int k, int n1, int n2) {
 return(a[n2] == n1);
}

/*@ requires n > 0 && \valid(a+(0..n-1));
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 <==> is_eq_im(a,n,k,n1);*/
int isEqIm(int a[], int n, int k, int n1) {
 int i;

 /*@ loop invariant 0 <= i <= n;
   @ loop invariant ! is_eq_im_gen(a,n,k,n1,i);
   @ loop assigns i;
   @ loop variant n-i;*/
 for (i = 0; i < n; i++)
  if (isX3(a,n,k,n1,i) == 1) return 1;
 return 0;
}

/*@ requires n >= k > 0 && \valid(a+(0..n-1));
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 <==> is_im(a,n,k);*/
int isIm(int a[], int n, int k) {
 int i;

 /*@ loop invariant 0 <= i <= k;
   @ loop invariant is_im_gen(a,n,k,i);
   @ loop assigns i;
   @ loop variant k-i;*/
 for (i = 0; i < k; i++)
  if (isEqIm(a,n,k,i) == 0) return 0;
 return 1;
}

/* Returns 1 when the array a is a surjection, and 0 otherwise. */
/*@ requires n >= k > 0 && \valid(a+(0..n-1));
  @ requires is_fct(a,n,k);
  @ assigns \nothing;
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_surj(a,n,k); */
int isSurj(int a[], int n, int k) {
 return (isIm(a,n,k));
}

/*@ requires n >= k > 0 && \valid(a+(0..n-1));
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_surj(a,n,k); */
int firstSurj(int a[], int n, int k) {
 int tmp;
 
 tmp = firstFct(a,n,k);
 /*@ loop invariant tmp != 0 ==> is_fct(a,n,k);
   @ loop assigns a[0..n-1],tmp; */
 while (tmp != 0 && isSurj(a,n,k) == 0) {
  tmp = nextFct(a,n,k);
 } 
 if (tmp == 0) { return 0; }
 return 1;
}

/*@ requires n >= k > 0 && \valid(a+(0..n-1));
  @ requires is_surj(a,n,k);
  @ assigns a[0..n-1];
  @ ensures \result == 0 || \result == 1;
  @ ensures \result == 1 ==> is_surj(a,n,k); */
int nextSurj(int a[], int n, int k) {
 int tmp;
 
 /*@ loop invariant is_fct(a,n,k);
   @ loop assigns a[0..n-1],tmp; */
 do {
  tmp = nextFct(a,n,k);
 } while (tmp != 0 && isSurj(a,n,k) == 0);
 if (tmp == 0) { return 0; }
 return 1;
}
